import * as React from 'react';
import { Errors, FormHookContext } from './types';
export declare const formContext: React.Context<FormHookContext>;
export interface SuccessBag {
    resetForm: () => void;
}
export interface ErrorBag {
    setErrors: (errors: Errors) => void;
    setFormError: (error: string) => void;
}
export interface CallBag {
    props?: object;
    setErrors: (errors: Errors) => void;
    setFormError: (error: string) => void;
}
export interface Payload {
    change: (fieldId: string, value: any) => void;
    formError?: string | null;
    isDirty?: boolean | null;
    isSubmitting?: boolean | null;
    handleSubmit: (e?: React.SyntheticEvent) => void;
    resetForm: () => void;
}
export interface FormOptions<T> extends Omit<React.HTMLProps<HTMLFormElement>, 'onSubmit' | 'onError'> {
    children?: ((form: Payload) => React.ReactNode) | React.ReactNode;
    enableReinitialize?: boolean;
    initialErrors?: Errors;
    initialValues?: Partial<T>;
    noForm?: boolean;
    onError?: (error: Error, callbag: ErrorBag) => void;
    onSuccess?: (result: any, callbag: SuccessBag) => void;
    onSubmit: (values: Partial<T>, callbag: CallBag) => Promise<any> | any;
    shouldSubmitWhenInvalid?: boolean;
    validate?: (values: Partial<T>) => object | undefined;
    validateOnBlur?: boolean;
    validateOnChange?: boolean;
}
declare function Form<Values extends object>({ children, enableReinitialize, initialErrors, initialValues, onSubmit, noForm, validate, onError, onSuccess, shouldSubmitWhenInvalid, validateOnBlur, validateOnChange, ...formProps }: FormOptions<Values>): JSX.Element;
export default Form;
