(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
  (global = global || self, factory(global.hookedForm = {}, global.react));
}(this, (function (exports, React) {
  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  function deriveInitial(value, defaultValue) {
    var result = {};

    for (var key in value) {
      if (Array.isArray(value[key])) {
        result[key] = value[key].map(function (val) {
          return val && typeof val === 'object' ? deriveInitial(val, defaultValue) : defaultValue;
        });
      } else if (value[key] && typeof value[key] === 'object') {
        result[key] = deriveInitial(value[key], defaultValue);
      } else {
        result[key] = defaultValue;
      }
    }

    return result;
  }

  function get(source, key) {
    var path = key.replace(/\[("|')?([^\[\]]+)\1\]/g, '.$2').split('.'),
        index = 0;

    while (source && path.length > index) {
      source = source[path[index++]];
    }

    return source;
  }
  function set(source, key, value) {
    return setHelper(source, value, key.replace(/\[("|')?([^\[\]]+)\1\]/g, '.$2').split('.'), 0);
  }

  function setHelper(source, value, pathArray, currentIndex) {
    var _Object$assign;

    if (currentIndex >= pathArray.length) return value;
    var continuedPath = setHelper(source && source[pathArray[currentIndex]], value, pathArray, currentIndex + 1);

    if (!source) {
      if (isNaN(pathArray[currentIndex])) {
        var _ref;

        return _ref = {}, _ref[pathArray[currentIndex]] = continuedPath, _ref;
      }

      var result = [];
      result[pathArray[currentIndex]] = continuedPath;
      return result;
    }

    if (Array.isArray(source)) {
      source[pathArray[currentIndex]] = continuedPath;
      return source;
    }

    return Object.assign({}, source, (_Object$assign = {}, _Object$assign[pathArray[currentIndex]] = continuedPath, _Object$assign));
  }

  var EMPTY_OBJ = {};
  var formContext = React.createContext(EMPTY_OBJ);

  function Form(_ref) {
    var children = _ref.children,
        enableReinitialize = _ref.enableReinitialize,
        initialErrors = _ref.initialErrors,
        initialValues = _ref.initialValues,
        onSubmit = _ref.onSubmit,
        noForm = _ref.noForm,
        validate = _ref.validate,
        onError = _ref.onError,
        onSuccess = _ref.onSuccess,
        shouldSubmitWhenInvalid = _ref.shouldSubmitWhenInvalid,
        validateOnBlur = _ref.validateOnBlur,
        validateOnChange = _ref.validateOnChange,
        formProps = _objectWithoutPropertiesLoose(_ref, ["children", "enableReinitialize", "initialErrors", "initialValues", "onSubmit", "noForm", "validate", "onError", "onSuccess", "shouldSubmitWhenInvalid", "validateOnBlur", "validateOnChange"]);

    var fieldValidators = React.useRef([]);
    var isDirty = React.useRef(false);

    var _React$useState = React.useState(initialValues || EMPTY_OBJ),
        values = _React$useState[0],
        setValues = _React$useState[1];

    var _React$useState2 = React.useState(initialErrors && function () {
      return deriveInitial(initialErrors, true);
    } || EMPTY_OBJ),
        touched = _React$useState2[0],
        setTouched = _React$useState2[1];

    var _React$useState3 = React.useState(initialErrors || EMPTY_OBJ),
        errors = _React$useState3[0],
        _setErrors = _React$useState3[1];

    var submittingState = React.useState(false);
    var formErrorState = React.useState();

    var validateForm = function validateForm() {
      var validationErrors = validate && validate(values) || EMPTY_OBJ;
      fieldValidators.current.some(function (tuple) {
        var error = tuple[1](get(values, tuple[0]));

        if (error) {
          validationErrors = set(validationErrors, tuple[0], error);
        }
      });
      if (fieldValidators.current.length) validationErrors = JSON.parse(JSON.stringify(validationErrors));

      if (validationErrors !== errors && JSON.stringify(errors) !== JSON.stringify(validationErrors)) {
        _setErrors(validationErrors);
      }

      return validationErrors;
    };

    var resetForm = function resetForm() {
      isDirty.current = false;
      setValues(initialValues || EMPTY_OBJ);

      if (initialErrors) {
        setTouched(deriveInitial(initialErrors, true));

        _setErrors(initialErrors);
      }
    };

    var handleSubmit = function handleSubmit(event) {
      if (event && event.preventDefault) event.preventDefault();
      submittingState[1](true);
      var fieldErrors = validateForm();
      setTouched(deriveInitial(fieldErrors, true));

      if (!shouldSubmitWhenInvalid && Object.keys(fieldErrors).length > 0) {
        return submittingState[1](false);
      }

      return new Promise(function (resolve) {
        return resolve(onSubmit(values, {
          setErrors: function setErrors(submitErrors) {
            _setErrors(submitErrors);
          },
          setFormError: function setFormError(err) {
            formErrorState[1](err);
          }
        }));
      }).then(function (result) {
        submittingState[1](false);
        if (onSuccess) onSuccess(result, {
          resetForm: resetForm
        });
      }, function (err) {
        submittingState[1](false);
        if (onError) onError(err, {
          setErrors: function setErrors(submitErrors) {
            _setErrors(submitErrors);
          },
          setFormError: function setFormError(err) {
            formErrorState[1](err);
          }
        });
      });
    };

    React.useEffect(function () {
      if (enableReinitialize) resetForm();
    }, [initialValues]);
    React.useEffect(function () {
      if ((validateOnBlur === undefined || validateOnChange || validateOnBlur) && isDirty.current) {
        validateForm();
      }
    }, [validateOnBlur === undefined ? touched : validateOnBlur && touched, validateOnChange && values, isDirty.current]);

    var change = function change(fieldId, value) {
      isDirty.current = true;
      setValues(function (state) {
        return set(state, fieldId, value);
      });
    };

    var toRender = typeof children === 'function' ? children({
      change: change,
      formError: formErrorState[0],
      isDirty: isDirty.current,
      isSubmitting: submittingState[0],
      handleSubmit: handleSubmit,
      resetForm: resetForm
    }) : children;
    return React.createElement(formContext.Provider, {
      value: {
        errors: errors,
        formError: formErrorState[0],
        isDirty: isDirty.current,
        isSubmitting: submittingState[0],
        resetForm: resetForm,
        setFieldError: function setFieldError(fieldId, error) {
          _setErrors(function (state) {
            return set(state, fieldId, error);
          });
        },
        setFieldTouched: function setFieldTouched(fieldId, value) {
          setTouched(function (state) {
            return set(state, fieldId, value);
          });
        },
        setFieldValue: change,
        submit: handleSubmit,
        touched: touched,
        validate: validateForm,
        values: values,
        _fieldValidators: fieldValidators.current
      }
    }, noForm ? toRender : React.createElement("form", Object.assign({
      onSubmit: handleSubmit
    }, formProps), toRender));
  }

  function useField(fieldId, validate) {
    if (process.env.NODE_ENV !== 'production' && (!fieldId || typeof fieldId !== 'string')) {
      throw new Error('The Field needs a valid "fieldId" property to function correctly.');
    }

    var ctx = React.useContext(formContext);
    var isMounted = React.useRef(false);
    React.useEffect(function () {
      var tuple = [fieldId, validate];

      if (validate) {
        ctx._fieldValidators.push(tuple);

        if (isMounted.current) {
          var newError = validate(get(ctx.values, fieldId));

          if (newError !== get(ctx.errors, fieldId)) {
            ctx.setFieldError(fieldId, newError);
          }
        }
      } else if (get(ctx.errors, fieldId)) {
        ctx.setFieldError(fieldId, undefined);
      }

      isMounted.current = true;
      return function () {
        if (validate) {
          ctx._fieldValidators.splice(ctx._fieldValidators.indexOf(tuple), 1);
        }
      };
    }, [fieldId, validate]);
    return [{
      onBlur: function onBlur() {
        ctx.setFieldTouched(fieldId, true);
      },
      onChange: function onChange(value) {
        ctx.setFieldValue(fieldId, value);
      },
      onFocus: function onFocus() {
        ctx.setFieldTouched(fieldId, false);
      }
    }, {
      error: get(ctx.errors, fieldId),
      touched: get(ctx.touched, fieldId),
      value: get(ctx.values, fieldId) || ''
    }];
  }

  function useFieldArray(fieldId) {
    if (process.env.NODE_ENV !== 'production' && (!fieldId || typeof fieldId !== 'string')) {
      throw new Error('The FieldArray needs a valid "fieldId" property to function correctly.');
    }

    var ctx = React.useContext(formContext);
    var value = get(ctx.values, fieldId) || [];
    return [{
      add: function add(element) {
        ctx.setFieldValue(fieldId, [].concat(value, [element]));
      },
      insert: function insert(at, element) {
        var touched = get(ctx.touched, fieldId) || [];
        var errors = get(ctx.errors, fieldId);
        value.splice(at, 0, element);
        touched.splice(at, 0, false);
        ctx.setFieldValue(fieldId, value);
        ctx.setFieldTouched(fieldId, touched);

        if (errors) {
          errors.splice(at, 0, undefined);
          ctx.setFieldError(fieldId, errors);
        }
      },
      remove: function remove(index) {
        var touched = get(ctx.touched, fieldId) || [];
        var errors = get(ctx.errors, fieldId);
        value.splice(index, 1);
        touched.splice(index, 1);
        ctx.setFieldValue(fieldId, value);
        ctx.setFieldTouched(fieldId, touched);

        if (errors) {
          errors.splice(index, 1);
          ctx.setFieldError(fieldId, errors);
        }
      }
    }, {
      error: get(ctx.errors, fieldId),
      value: value
    }];
  }

  function useError(fieldId) {
    if (process.env.NODE_ENV !== 'production' && (!fieldId || typeof fieldId !== 'string')) {
      throw new Error('The Error needs a valid "fieldId" property to function correctly.');
    }

    return get(React.useContext(formContext).errors, fieldId);
  }

  var useFormConnect = (function () {
    return React.useContext(formContext);
  });

  function OptionsContainer(options) {
    var isMounted = false;
    return function FormOuterWrapper(Component) {
      return function FormWrapper(props) {
        var initialValuesState = React.useState(function () {
          return options.mapPropsToValues && options.mapPropsToValues(props);
        });
        React.useEffect(function () {
          if (options.enableReinitialize && options.mapPropsToValues && isMounted) initialValuesState[1](options.mapPropsToValues(props));
          isMounted = true;
        }, options.enableReinitialize ? Object.values(props) : []);
        return React.createElement(Form, Object.assign({}, options, {
          initialValues: initialValuesState[0] || options.initialValues,
          noForm: true,
          validateOnBlur: options.validateOnBlur === undefined ? false : options.validateOnBlur
        }), function (form) {
          return React.createElement(Component, Object.assign({}, form, props));
        });
      };
    };
  }

  function useSpy(fieldId, cb) {
    var ctx = React.useContext(formContext);
    var isMounted = React.useRef(false);
    var prev = React.useRef();
    var value = get(ctx.values, fieldId);
    React.useEffect(function () {
      if (isMounted.current && cb && prev.current !== value) cb(value, ctx);
      isMounted.current = true;
      prev.current = value;
    }, [value]);
    return {
      error: get(ctx.errors, fieldId),
      touched: get(ctx.touched, fieldId),
      value: value
    };
  }

  exports.Form = OptionsContainer;
  exports.HookedForm = Form;
  exports.useError = useError;
  exports.useField = useField;
  exports.useFieldArray = useFieldArray;
  exports.useFormConnect = useFormConnect;
  exports.useSpy = useSpy;

})));
//# sourceMappingURL=hooked-form.umd.js.map
